/*
    *
    * Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
    *
    * This file is a part of DesQ project <https://gitlab.com/desq/>
    *

    *
    * This file was originally taken from LXQt project
    * <https://github.com/lxqt/lxqt-panel>
    * Several modifications have been made to suit the needs of DesQ
    * Any and all bug reports are to be filed with DesQ and not LXQt
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#pragma once

#include <functional>
#include "SNIInterface.hpp"

template<typename>
    struct remove_class_type { using type = void; };

template<typename C, typename R, typename... ArgTypes>
    struct  remove_class_type<R (C::*)(ArgTypes...)> { using type = R(ArgTypes...); };

template<typename C, typename R, typename... ArgTypes>
    struct remove_class_type<R (C::*)(ArgTypes...) const> { using type = R(ArgTypes...); };

template <typename L>
class call_sig_helper {
    template <typename L1>
        static decltype(&L1::operator()) test(int);

    template <typename L1>
        static void test(...); //bluff

    public:
        using type = decltype(test<L>(0));
};

template <typename L>
    struct call_signature : public remove_class_type<typename call_sig_helper<L>::type> {};

template <typename R, typename... ArgTypes>
    struct call_signature<R (ArgTypes...)> { using type = R (ArgTypes...); };

template <typename R, typename... ArgTypes>
    struct call_signature<R (*)(ArgTypes...)> { using type = R (ArgTypes...); };

template <typename C, typename R, typename... ArgTypes>
    struct call_signature<R (C::*)(ArgTypes...)> { using type = R (ArgTypes...); };

template<typename C, typename R, typename... ArgTypes>
    struct call_signature<R (C::*)(ArgTypes...) const>  { using type = R(ArgTypes...); };

template <typename>
    struct is_valid_signature : public std::false_type {};

template <typename Arg>
    struct is_valid_signature<void (Arg)> : public std::true_type {};

class SniAsync : public QObject {
    Q_OBJECT;

    public:
        SniAsync( QString, QString, QDBusConnection, QObject *parent = nullptr );

        template <typename F>
        void propertyGetAsync( QString const &name, F finished ) {
            static_assert(
                is_valid_signature<typename call_signature<F>::type>::value, "need callable ( lambda, *function, callable obj ) ( Arg ) -> void"
            );

            connect(
                new QDBusPendingCallWatcher{ asyncPropGet( name ), this },
                &QDBusPendingCallWatcher::finished,
                [this, finished, name]( QDBusPendingCallWatcher * call ) {

                    QDBusPendingReply<QVariant> reply = *call;
                    if ( reply.isError() )
                        qDebug() << "Error on DBus request:" << reply.error();

                    finished( qdbus_cast<typename std::function<typename call_signature<F>::type>::argument_type>( reply.value() ) );
                    call->deleteLater();
                }
            );
        };

        // exposed methods from StatusNotifierItem
        inline QString service() const {
            return mSni->service();
        }

    public slots:
        // Forwarded slots from StatusNotifierItem
        QDBusPendingReply<> Activate(int x, int y);
        QDBusPendingReply<> ContextMenu(int x, int y);
        QDBusPendingReply<> Scroll(int delta, const QString &orientation);
        QDBusPendingReply<> SecondaryActivate(int x, int y);

    signals:
        // Forwarded signals from StatusNotifierItem
        void NewAttentionIcon();
        void NewIcon();
        void NewOverlayIcon();
        void NewStatus(const QString &status);
        void NewTitle();
        void NewToolTip();

    private:
        QDBusPendingReply<QDBusVariant> asyncPropGet(QString const & property);

    private:
        StatusNotifierItem *mSni;
};
