/*
    *
    * Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
    *
    * This file is a part of DesQ project <https://gitlab.com/desq/>
    *

    *
    * This file was originally taken from LXQt project
    * <https://github.com/lxqt/lxqt-panel>
    * Several modifications have been made to suit the needs of DesQ
    * Any and all bug reports are to be filed with DesQ and not LXQt
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#pragma once

#include <QDBusConnection>
#include <QDBusContext>
#include <QDBusMessage>
#include <QDBusMetaType>
#include <QDBusServiceWatcher>

class StatusNotifierWatcher : public QObject, protected QDBusContext
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.StatusNotifierWatcher")
    Q_SCRIPTABLE Q_PROPERTY(bool IsStatusNotifierHostRegistered READ isStatusNotifierHostRegistered)
    Q_SCRIPTABLE Q_PROPERTY(int ProtocolVersion READ protocolVersion)
    Q_SCRIPTABLE Q_PROPERTY(QStringList RegisteredStatusNotifierItems READ RegisteredStatusNotifierItems)

    public:
        explicit StatusNotifierWatcher(QObject *parent = nullptr);
        ~StatusNotifierWatcher();

        bool isStatusNotifierHostRegistered() { return mHosts.count() > 0; }
        int protocolVersion() const { return 0; }
        QStringList RegisteredStatusNotifierItems() const { return mServices; }

        bool isServiceRunning();
        bool isObjectRegistered();

    signals:
        Q_SCRIPTABLE void StatusNotifierItemRegistered(const QString &service);
        Q_SCRIPTABLE void StatusNotifierItemUnregistered(const QString &service);
        Q_SCRIPTABLE void StatusNotifierHostRegistered();

    public slots:
        Q_SCRIPTABLE void RegisterStatusNotifierItem(const QString &serviceOrPath);
        Q_SCRIPTABLE void RegisterStatusNotifierHost(const QString &service);

        void serviceUnregistered(const QString &service);

    private:
        QStringList mServices;
        QStringList mHosts;
        QDBusServiceWatcher *mWatcher;

    	bool serviceRunning;
    	bool objectRegistered;
};
